"use strict";

exports.__esModule = true;
exports.setDeviceWidth = setDeviceWidth;
exports.setViewportWidth = setViewportWidth;
exports.default = void 0;

var DriverDOM = _interopRequireWildcard(require("driver-dom"));

var _styleUnit = require("style-unit");

var Flexbox = _interopRequireWildcard(require("./flexbox"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var STYLE = 'style';
var DEFAULT_VIEWPORT = 750;
var deviceWidth = null;
var viewportWidth = null;
var driver = Object.assign({}, DriverDOM, {
  createElement: function createElement(type, props, component) {
    if (props.hasOwnProperty(STYLE)) {
      var _extends2;

      var style = props[STYLE];
      var node = DriverDOM.createElement(type, _extends({}, props, (_extends2 = {}, _extends2[STYLE] = null, _extends2)), component);
      driver.setStyle(node, style);
      return node;
    } else {
      return DriverDOM.createElement(type, props, component);
    }
  },
  addEventListener: function addEventListener(node, eventName, eventHandler, props) {
    eventName = normalizeEventName(node, eventName, props);
    return DriverDOM.addEventListener(node, eventName, eventHandler, props);
  },
  removeEventListener: function removeEventListener(node, eventName, eventHandler, props) {
    eventName = normalizeEventName(node, eventName, props);
    return DriverDOM.removeEventListener(node, eventName, eventHandler, props);
  },
  beforeRender: function beforeRender(options) {
    // Init rem unit
    (0, _styleUnit.setRem)(getDeviceWidth() / getViewportWidth());
    return DriverDOM.beforeRender(options);
  },
  setStyle: function setStyle(node, style) {
    var tranformedStyle = {};

    if (Array.isArray(style)) {
      style = style.reduce(function (prev, curr) {
        return Object.assign(prev, curr);
      }, {});
    }

    for (var prop in style) {
      if (style.hasOwnProperty(prop)) {
        var val = style[prop];

        if (Flexbox.isFlexProp(prop)) {
          Flexbox[prop](val, tranformedStyle);
        } else {
          tranformedStyle[prop] = (0, _styleUnit.convertUnit)(val, prop);
        }
      }
    }

    for (var _prop in tranformedStyle) {
      if (tranformedStyle.hasOwnProperty(_prop)) {
        var transformValue = tranformedStyle[_prop]; // Hack handle compatibility issue

        if (Array.isArray(transformValue)) {
          for (var i = 0; i < transformValue.length; i++) {
            node.style[_prop] = transformValue[i];
          }
        } else {
          node.style[_prop] = transformValue;
        }
      }
    }
  }
});

function normalizeEventName(node, eventName, props) {
  var tagName = node.tagName.toLowerCase();

  if (eventName === 'change' && (tagName === 'textarea' || tagName === 'input' && (!props.type || props.type === 'text' || props.type === 'password'))) {
    eventName = 'input';
  } else if (eventName === 'doubleclick') {
    eventName = 'dblclick';
  }

  return eventName;
}

function getDeviceWidth() {
  return deviceWidth || typeof DEVICE_WIDTH !== 'undefined' && DEVICE_WIDTH || getClientWidth();
}

function getClientWidth() {
  return document.documentElement.clientWidth;
}

function setDeviceWidth(width) {
  deviceWidth = width;
}

function getViewportWidth() {
  return viewportWidth || typeof VIEWPORT_WIDTH !== 'undefined' && VIEWPORT_WIDTH || DEFAULT_VIEWPORT;
}

function setViewportWidth(width) {
  viewportWidth = width;
}

var _default = driver;
exports.default = _default;